#ifndef _DEFINES_H_
#define _DEFINES_H_
#include <cmath>

#define sqr(x) ((x)*(x))
#define pow3(x) ((x)*(x)*(x))
#define M_PI 3.1415926

typedef int Vec2i[2];
typedef float Vec2f[2];
typedef double Vec2d[2];

typedef unsigned char Vec3c[3];
typedef short int Vec3s[3]; 
typedef int Vec3i[3]; 
typedef float Vec3f[3];
typedef double Vec3d[3];

typedef float Mat3f[3][3];
typedef double Mat3d[3][3];
typedef double SymMat3d[6];

const int _symindex[3]={0,1,3};

#define SYM(m,a,b) ((a<b)?(m)[a+_symindex[b]]:(m)[b+_symindex[a]])

#define VecZero(a) { (a)[0]=0; (a)[1]=0; (a)[2]=0; }

#define MatZero(a) { (a)[0][0]=0; (a)[0][1]=0; (a)[0][2]=0; (a)[1][0]=0; (a)[1][1]=0; (a)[1][2]=0; (a)[2][0]=0; (a)[2][1]=0; (a)[2][2]=0; }

#define MatVecMul(A, x, b) { b[0]=A[0][0]*x[0]+A[0][1]*x[1]+A[0][2]*x[2]; \
							 b[1]=A[1][0]*x[0]+A[1][1]*x[1]+A[1][2]*x[2]; \
							 b[2]=A[2][0]*x[0]+A[2][1]*x[1]+A[2][2]*x[2]; }
//adv
#define MatVecMul4(A, x, b){ b[0]=A[0][0]*x[0]+A[0][1]*x[1]+A[0][2]*x[2]+A[0][3]*x[3]; \
							 b[1]=A[1][0]*x[0]+A[1][1]*x[1]+A[1][2]*x[2]+A[1][3]*x[3]; \
							 b[2]=A[2][0]*x[0]+A[2][1]*x[1]+A[2][2]*x[2]+A[2][3]*x[3]; \
							 b[3]=A[3][0]*x[0]+A[3][1]*x[1]+A[3][2]*x[2]+A[3][3]*x[3]; }

#define MatVecMul4Part2(A, x, b)	{ b[0]=A[0]*x[0]+A[1]*x[1]+A[2]*x[2]+A[3]*x[3]; \
									b[1]=A[5]*x[0]+A[6]*x[1]+A[7]*x[2]+A[8]*x[3]; \
									b[2]=A[9]*x[0]+A[10]*x[1]+A[11]*x[2]+A[12]*x[3]; \
									b[3]=A[13]*x[0]+A[14]*x[1]+A[15]*x[2]+A[16]*x[3]; }

#define VecMatMul(x, A, b) { b[0]=A[0][0]*x[0]+A[1][0]*x[1]+A[2][0]*x[2]; \
						  	 b[1]=A[0][1]*x[0]+A[1][1]*x[1]+A[2][1]*x[2]; \
							 b[2]=A[0][2]*x[0]+A[1][2]*x[1]+A[2][2]*x[2]; }

#define VecSub(a, b, c) { (c)[0]=(a)[0]-(b)[0]; (c)[1]=(a)[1]-(b)[1]; (c)[2]=(a)[2]-(b)[2]; }

#define VecAdd(a, b, c) { (c)[0]=(a)[0]+(b)[0]; (c)[1]=(a)[1]+(b)[1]; (c)[2]=(a)[2]+(b)[2]; }

#define VecScaledAdd(a, b, c) { c[0]+=(a)*b[0]; c[1]+=(a)*b[1]; c[2]+=(a)*b[2]; }

#define VecAssign(a, b) { (b)[0]=(a)[0]; (b)[1]=(a)[1]; (b)[2]=(a)[2]; }

#define VecScale(s, a, b) { (b)[0]=(a)[0]*(s); (b)[1]=(a)[1]*(s); (b)[2]=(a)[2]*(s); }

#define SymVecVecT(a,b,C) { SYM(C,0,0)=(a)[0]*(b)[0]; \
                            SYM(C,0,1)=(a)[0]*(b)[1]; \
                            SYM(C,0,2)=(a)[0]*(b)[2]; \
                            SYM(C,1,1)=(a)[1]*(b)[1]; \
                            SYM(C,1,2)=(a)[1]*(b)[2]; \
                            SYM(C,2,2)=(a)[2]*(b)[2]; \
}

#define SymMatScaledAdd(a, b, c) { c[0]+=(a)*b[0]; \
							       c[1]+=(a)*b[1]; \
                                   c[2]+=(a)*b[2]; \
								   c[3]+=(a)*b[3]; \
								   c[4]+=(a)*b[4]; \
                                   c[5]+=(a)*b[5]; \
}

#define MatScaledAdd(a, b, c) { (c)[0][0]+=(a)*(b)[0][0]; \
							    (c)[0][1]+=(a)*(b)[0][1]; \
                                (c)[0][2]+=(a)*(b)[0][2]; \
								(c)[1][0]+=(a)*(b)[1][0]; \
								(c)[1][1]+=(a)*(b)[1][1]; \
                                (c)[1][2]+=(a)*(b)[1][2]; \
								(c)[2][0]+=(a)*(b)[2][0]; \
								(c)[2][1]+=(a)*(b)[2][1]; \
                                (c)[2][2]+=(a)*(b)[2][2]; \
}

#define VecVecT(a,b,C) { (C)[0][0]=(a)[0]*(b)[0]; \
                         (C)[0][1]=(a)[0]*(b)[1]; \
                         (C)[0][2]=(a)[0]*(b)[2]; \
                         (C)[1][0]=(a)[1]*(b)[0]; \
                         (C)[1][1]=(a)[1]*(b)[1]; \
                         (C)[1][2]=(a)[1]*(b)[2]; \
                         (C)[2][0]=(a)[2]*(b)[0]; \
                         (C)[2][1]=(a)[2]*(b)[1]; \
                         (C)[2][2]=(a)[2]*(b)[2]; \
}

#define MatScale(s, A, B) { B[0][0]=s*A[0][0]; \
							B[0][1]=s*A[0][1]; \
							B[0][2]=s*A[0][2]; \
							B[1][0]=s*A[1][0]; \
							B[1][1]=s*A[1][1]; \
							B[1][2]=s*A[1][2]; \
							B[2][0]=s*A[2][0]; \
							B[2][1]=s*A[2][1]; \
							B[2][2]=s*A[2][2]; \
}

#define SymMatScale(s, A, B) { B[0]=s*A[0]; \
						       B[1]=s*A[1]; \
							   B[2]=s*A[2]; \
							   B[3]=s*A[3]; \
							   B[4]=s*A[4]; \
							   B[5]=s*A[5]; \
}

#define VecCross(a, b, c) {	c[0]=a[1]*b[2]-a[2]*b[1]; \
							c[1]=a[2]*b[0]-a[0]*b[2]; \
							c[2]=a[0]*b[1]-a[1]*b[0]; }

#define VecRotateUp(a) { temp=a[0]; a[0]=a[2]; a[2]=a[1]; a[1]=temp; }

#define VecRotateDown(a) { temp=a[0]; a[0]=a[1]; a[1]=a[2]; a[2]=temp; }

#define VecNormalize(a) { \
	double len=VecLen(a); \
	VecScale(1/len,a,a); \
}

#define MatMatMul(A, B, C) { C[0][0]=A[0][0]*B[0][0]+A[0][1]*B[1][0]+A[0][2]*B[2][0]; \
							 C[0][1]=A[0][0]*B[0][1]+A[0][1]*B[1][1]+A[0][2]*B[2][1]; \
							 C[0][2]=A[0][0]*B[0][2]+A[0][1]*B[1][2]+A[0][2]*B[2][2]; \
							 C[1][0]=A[1][0]*B[0][0]+A[1][1]*B[1][0]+A[1][2]*B[2][0]; \
							 C[1][1]=A[1][0]*B[0][1]+A[1][1]*B[1][1]+A[1][2]*B[2][1]; \
							 C[1][2]=A[1][0]*B[0][2]+A[1][1]*B[1][2]+A[1][2]*B[2][2]; \
							 C[2][0]=A[2][0]*B[0][0]+A[2][1]*B[1][0]+A[2][2]*B[2][0]; \
							 C[2][1]=A[2][0]*B[0][1]+A[2][1]*B[1][1]+A[2][2]*B[2][1]; \
							 C[2][2]=A[2][0]*B[0][2]+A[2][1]*B[1][2]+A[2][2]*B[2][2]; \
}

//adv
/*
#define MatMatMul4(A, B, C){ C[0][0]=A[0][0]*B[0][0]+A[0][1]*B[1][0]+A[0][2]*B[2][0]+A[0][3]*B[3][0]; \
							 C[0][1]=A[0][0]*B[0][1]+A[0][1]*B[1][1]+A[0][2]*B[2][1]+A[0][3]*B[3][1]; \
							 C[0][2]=A[0][0]*B[0][2]+A[0][1]*B[1][2]+A[0][2]*B[2][2]+A[0][3]*B[3][2]; \
							 C[0][3]=A[0][0]*B[0][3]+A[0][1]*B[1][3]+A[0][2]*B[2][3]+A[0][3]*B[3][3]; \  //adv

							 C[1][0]=A[1][0]*B[0][0]+A[1][1]*B[1][0]+A[1][2]*B[2][0]+A[1][3]*B[3][0]; \
							 C[1][1]=A[1][0]*B[0][1]+A[1][1]*B[1][1]+A[1][2]*B[2][1]+A[1][3]*B[3][1]; \
							 C[1][2]=A[1][0]*B[0][2]+A[1][1]*B[1][2]+A[1][2]*B[2][2]+A[1][3]*B[3][2]; \
							 C[1][3]=A[1][0]*B[0][3]+A[1][1]*B[1][3]+A[1][2]*B[2][3]+A[1][3]*B[3][3]; \ //adv

							 C[2][0]=A[2][0]*B[0][0]+A[2][1]*B[1][0]+A[2][2]*B[2][0]+A[2][3]*B[3][0]; \
							 C[2][1]=A[2][0]*B[0][1]+A[2][1]*B[1][1]+A[2][2]*B[2][1]+A[2][3]*B[3][1]; \
							 C[2][2]=A[2][0]*B[0][2]+A[2][1]*B[1][2]+A[2][2]*B[2][2]+A[2][3]*B[3][2]; \
							 C[2][3]=A[2][0]*B[0][3]+A[2][1]*B[1][3]+A[2][2]*B[2][3]+A[2][3]*B[3][3]; \  //adv

							 C[3][0]=A[3][0]*B[0][0]+A[3][1]*B[1][0]+A[3][2]*B[2][0]+A[3][3]*B[3][0]; \
							 C[3][1]=A[3][0]*B[0][1]+A[3][1]*B[1][1]+A[3][2]*B[2][1]+A[3][3]*B[3][1]; \
							 C[3][2]=A[3][0]*B[0][2]+A[3][1]*B[1][2]+A[3][2]*B[2][2]+A[3][3]*B[3][2]; \
							 C[3][3]=A[3][0]*B[0][3]+A[3][1]*B[1][3]+A[3][2]*B[2][3]+A[3][3]*B[3][3]; \  //adv
}
*/

#define MatMatAdd(A, B, C) { C[0][0]=A[0][0]+B[0][0]; \
							 C[0][1]=A[0][1]+B[0][1]; \
							 C[0][2]=A[0][2]+B[0][2]; \
							 C[1][0]=A[1][0]+B[1][0]; \
							 C[1][1]=A[1][1]+B[1][1]; \
							 C[1][2]=A[1][2]+B[1][2]; \
							 C[2][0]=A[2][0]+B[2][0]; \
							 C[2][1]=A[2][1]+B[2][1]; \
							 C[2][2]=A[2][2]+B[2][2]; \
}

#define MatMatSub(A, B, C) { C[0][0]=A[0][0]-B[0][0]; \
							 C[0][1]=A[0][1]-B[0][1]; \
							 C[0][2]=A[0][2]-B[0][2]; \
							 C[1][0]=A[1][0]-B[1][0]; \
							 C[1][1]=A[1][1]-B[1][1]; \
							 C[1][2]=A[1][2]-B[1][2]; \
							 C[2][0]=A[2][0]-B[2][0]; \
							 C[2][1]=A[2][1]-B[2][1]; \
							 C[2][2]=A[2][2]-B[2][2]; \
}

#define SymMatSymMatAdd(A, B, C) { C[0]=A[0]+B[0]; C[1]=A[1]+B[1]; C[2]=A[2]+B[2]; \
                                   C[3]=A[3]+B[3]; C[4]=A[4]+B[4]; C[5]=A[5]+B[5]; \
}

#define SymMatSymMatSub(A, B, C) { C[0]=A[0]-B[0]; C[1]=A[1]-B[1]; C[2]=A[2]-B[2]; \
                                   C[3]=A[3]-B[3]; C[4]=A[4]-B[4]; C[5]=A[5]-B[5]; \
}

#define MatSymMatAdd(A, B, C) { C[0][0]=A[0][0]+SYM(B,0,0); \
								C[0][1]=A[0][1]+SYM(B,0,1); \
								C[0][2]=A[0][2]+SYM(B,0,2); \
								C[1][0]=A[1][0]+SYM(B,1,0); \
								C[1][1]=A[1][1]+SYM(B,1,1); \
								C[1][2]=A[1][2]+SYM(B,1,2); \
								C[2][0]=A[2][0]+SYM(B,2,0); \
								C[2][1]=A[2][1]+SYM(B,2,1); \
								C[2][2]=A[2][2]+SYM(B,2,2); \
};

#define MatSymMatSub(A, B, C) { C[0][0]=A[0][0]-SYM(B,0,0); \
								C[0][1]=A[0][1]-SYM(B,0,1); \
								C[0][2]=A[0][2]-SYM(B,0,2); \
								C[1][0]=A[1][0]-SYM(B,1,0); \
								C[1][1]=A[1][1]-SYM(B,1,1); \
								C[1][2]=A[1][2]-SYM(B,1,2); \
								C[2][0]=A[2][0]-SYM(B,2,0); \
								C[2][1]=A[2][1]-SYM(B,2,1); \
								C[2][2]=A[2][2]-SYM(B,2,2); \
};

#define MatMatTMul(A, B, C) { C[0][0]=A[0][0]*B[0][0]+A[0][1]*B[0][1]+A[0][2]*B[0][2]; \
				 		      C[0][1]=A[0][0]*B[1][0]+A[0][1]*B[1][1]+A[0][2]*B[1][2]; \
							  C[0][2]=A[0][0]*B[2][0]+A[0][1]*B[2][1]+A[0][2]*B[2][2]; \
							  C[1][0]=A[1][0]*B[0][0]+A[1][1]*B[0][1]+A[1][2]*B[0][2]; \
							  C[1][1]=A[1][0]*B[1][0]+A[1][1]*B[1][1]+A[1][2]*B[1][2]; \
							  C[1][2]=A[1][0]*B[2][0]+A[1][1]*B[2][1]+A[1][2]*B[2][2]; \
							  C[2][0]=A[2][0]*B[0][0]+A[2][1]*B[0][1]+A[2][2]*B[0][2]; \
							  C[2][1]=A[2][0]*B[1][0]+A[2][1]*B[1][1]+A[2][2]*B[1][2]; \
							  C[2][2]=A[2][0]*B[2][0]+A[2][1]*B[2][1]+A[2][2]*B[2][2]; \
}

#define VecDot(a, b) ( (a)[0]*(b)[0]+(a)[1]*(b)[1]+(a)[2]*(b)[2] )

#define VecSqr(a) (sqr(a[0])+sqr(a[1])+sqr(a[2]))

#define MatAssign(A, B) { (B)[0][0]=(A)[0][0]; (B)[0][1]=(A)[0][1]; (B)[0][2]=(A)[0][2]; \
						  (B)[1][0]=(A)[1][0]; (B)[1][1]=(A)[1][1]; (B)[1][2]=(A)[1][2]; \
						  (B)[2][0]=(A)[2][0]; (B)[2][1]=(A)[2][1]; (B)[2][2]=(A)[2][2]; \
}

#define VecLen(a) (sqrt((double)VecSqr(a))) 

#define VecRotateUp(a) { temp=a[0]; a[0]=a[2]; a[2]=a[1]; a[1]=temp; }

#define VecRotateDown(a) { temp=a[0]; a[0]=a[1]; a[1]=a[2]; a[2]=temp; }

//R = Rz.Ry.Rx
#define RotationMat(r, R) { \
	Vec3d tmpc; \
	Vec3d tmps; \
	tmpc[0]=cos(r[0]); tmpc[1]=cos(r[1]); tmpc[2]=cos(r[2]); \
	tmps[0]=sin(r[0]); tmps[1]=sin(r[1]); tmps[2]=sin(r[2]); \
	R[0][0]=tmpc[1]*tmpc[2]; \
	R[0][1]=tmpc[2]*tmps[0]*tmps[1]-tmpc[0]*tmps[2]; \
	R[0][2]=tmpc[0]*tmpc[2]*tmps[1]+tmps[0]*tmps[2]; \
	R[1][0]=tmpc[1]*tmps[2]; \
	R[1][1]=tmpc[0]*tmpc[2]+tmps[0]*tmps[1]*tmps[2]; \
	R[1][2]=-tmpc[2]*tmps[0]+tmpc[0]*tmps[1]*tmps[2]; \
	R[2][0]=-tmps[1]; \
	R[2][1]=tmpc[1]*tmps[0]; \
	R[2][2]=tmpc[0]*tmpc[1]; \
}

#define RotationVec(R, r) { \
	double cy = sqrt(sqr(R[0][0])+sqr(R[1][0])); \
    if (cy > 16*FLT_EPSILON) { \
		r[0] = atan2(R[2][1], R[2][2]); \
        r[1] = atan2(-R[2][0], cy); \
        r[2] = atan2(R[1][0], R[0][0]); \
    } else { \
        r[0] = atan2(-R[1][2], R[1][1]); \
        r[1] = atan2(-R[2][0], cy); \
        r[2] = 0; \
	} \
}

#endif