// PlyMesh.h: interface for the CPlyMesh class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PLYMESH_H__883ED3A4_BD40_41E8_85F8_326A79B967B9__INCLUDED_)
#define AFX_PLYMESH_H__883ED3A4_BD40_41E8_85F8_326A79B967B9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//This is a wrapped version of the plytest.c code written by Greg Turk
//#include "PlyModel\stdafx.h"
//#include "..\ply\ply-ext.h"
#include "..\ply\ply.h"

//added by adv on 6/1 from flowply
#define VERTEX_PROCESSED         1    // Vertex has already been queued, or should not be processed.
#define VALUE_ASSIGNED           2    // Vertex has been assigned a dtime (dz or color) value.
#define START_BOUNDARY           3    // Vertex is part of start boundary (time = start).
#define END_BOUNDARY             4    // Vertex is part of start boundary (time = start).
#define CORR_BOUNDARY            5    // Vertex is part of the initial correspondence boundary.

typedef struct sFace {
	//unsigned char cIntensity;  // this user attaches intensity to faces 
	unsigned char ucNumVerts;    // number of vertex indices in list 
	int *pi_verts;				 // vertex index list 
    void *other_props;           // other properties 

	//adv added from flowply
	int marked;
	int entering_edge;     // =0 for no entering edges, 
								//1 for edge 0/1, 2 for edge 1/2, 3 for edge 2/0. 
} Face;

typedef struct sVertex {
	float fx, fy, fz;         // the usual 3-space position of a vertex 
	float fr, fg, fb, fa;		  // color //fa added by adv from flowply
	float fnx, fny, fnz;	  // normal
	float s, t;				  // texture coordinates
    void *other_props;        // other properties 

	//adv added from flowply
	sFace* adj_tris[20];          // pointer to adjacent triangles, allow max of 20 triangles.  //ADV WAS JUST Face... NOW WHAT?

	int adj_verts[21];           // index of adjacent vertices, allow max of 21 vertices. 
	double adj_verts_w[21];      // weight that each adjacent vertex applies. 
	int valence;                 // triangle valence 
	int marked;                  // flag to indicate state of triangle in flow. 
	double max_dtime;            // max gradient in time direction. 
	int boundary_vertex;         // flag to indicate vertex is on a boundary, so vertex
									//	valence is one more than triangle valence. 
	double tx, ty, tz;           // x,y,z components of the tangent vector field. 
	double tconf;                // summation of tangents from surrounding trianlges gives a 
									//measure of confidence.  Small values mean tangent may
									//be near a source or sink point. 
	double gradx, grady, gradz;           // x,y,z components of the gradient vector field w.r.t. value.   //ADV WAS FX, FX, AND FZ
	double negtx, negty, negtz;
	double value[2];             // function value(s) to be assigned onto vertex to track 
									//correspondence.  For example, can use arc length
									//as a function. 
//adv these are going to have to be defined in plyMehs.cpp .. is that where I'll do 
	double vtx, vty, vtz;           // vector texture x,y, and z components added 6/10 
} Vertex;

class CPlyMesh  
{
public:
	CPlyMesh();
	~CPlyMesh();

	void ReadPlyMesh(char *szFileName);
	void ReadPlyMeshOld(char *szFileName); //added by adv 5/31/05
	void WritePlyMesh(char *szFileName);
	void WriteIVMesh(char *szFileName, bool bWriteBinaryFile = true);

	void collect_adj_info (); //added by adv on 6/01 from plyview /*will i need parameters? CPlyMesh is all verticies, but this method is for EACH one... */
	void boundary_corr_init(); //added by adv 6/1 from flowply to initialize points and set color(scalor field)?
	float desbrun();//added by adv 6/1 for geometric heat diffusion
	void assign_v3_v4_new(int v, sFace* t2, double* v3, double* v4); //added by adv on 6/1 because it is used by the method before it ... should these all be private???
	double area (sFace* tri); //added by adv on 6/1 from flowply, in the class
	//random things added from flowply by adv on 6/1 -- i thought not in class, but then decided to put them here
	void normalize_vec (double* vec);
	double dot (double* v1, double* v2);
	void cross (double* v1, double* v2, double* dest);
	double area (double* v1, double* v2);
	void average_normal (sFace** tris, int num_tris, sVertex** verts, int v, double* norm);

	void generate_normal_field(); //added by adv on 6/1 from flowply
	void generate_weighted_tangent_field();//added by adv on 6/1 from flowply
	void triangle_grad (sFace* tri, int v, double* g, double val0, double val1, double val2);//added by adv on 6/1 from flowply

	Vertex      **m_pVertexList;
    Face        **m_pFaceList;

	unsigned int	m_iNumVertices;
	unsigned int	m_iNumFaces;

private:
//	char		**m_pszElem_names; 
	int			m_iFileType;
	float		m_fVersion;
};

#endif // !defined(AFX_PLYMESH_H__883ED3A4_BD40_41E8_85F8_326A79B967B9__INCLUDED_)
